unit Unit1; (* Les tours de Hano en Delphi JC Meier d'aprs N. Rubenking   *)
            (* Solution visuelle donne par un programme en Turbo-Prolog    *)

            (* Towers of Hanoi in Delphi from N. Rubenking                 *)
            (* A visual solution is given by a Turbo-Prolog soft           *) 
            
interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Buttons, Spin;

type
  TForm1 = class(TForm)
    L: TShape;
    F: TShape;
    E: TShape;
    D: TShape;
    C: TShape;
    B: TShape;
    M: TShape;
    R: TShape;
    Label1: TLabel;
    Label_Essais: TLabel;
    BBQuitter: TBitBtn;
    Shape1: TShape;
    Shape2: TShape;
    Shape3: TShape;
    A: TShape;
    BDemarrer: TButton;
    SpinEdit1: TSpinEdit;
    Label3: TLabel;
    Bevel1: TBevel;
    BBAide: TBitBtn;
    BSolution: TButton;
    Bevel2: TBevel;
    Bevel3: TBevel;
    Image1: TImage;
    Image2: TImage;
    Timer1: TTimer;
    Bevel4: TBevel;
    BAbandon: TButton;
    procedure CDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure CDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure BDemarrerClick(Sender: TObject);
    procedure SpinEdit1Change(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Choisit_Niveau(Lequel : Byte);
    procedure Initialise_Hints;
    procedure BSolutionClick(Sender: TObject);
    procedure BBAideClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BAbandonClick(Sender: TObject);
  private
    { Dclarations private }
  public
    { Dclarations public }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

Const Nb_Essais    : Integer = 0;
      Jeu_Possible : Boolean = False;
      Nb_Disques   : Byte    = 6;

Var Dernier_Disque : Char;

(***  Solution astucieuse propose par  Neil Rubenking dans Delphi programming ***)
(*    for Dummies IDG Books : Delphi pour les nuls, une vritable mine d'or !    *)
(*    Le squelette du programme fait une vingtaine de lignes !                   *)
(*
  Les noms des tours :

                       A
                       B
                       C
                       D
                       E
                       F
                       L       M       R

 Mettre des hints  chaque disque :

 2 lettres pour chaque disque

 1re lettre  : nom du disque au-dessous
 2 me lettre : nom disque au-dessus

 X si rien au-dessous ou rien au-dessus

 au dmarrage :        A  :   BX     B en dessous, rien au dessus
                       B  :   CA     C en dessous, A au dessus
                       C  :   DB          "      "
                       D  :   EC
                       E  :   FD
                       F  :   LE
                       L  :   XF
                       M  :   XX
                       R  :   XX          "       "


Pour gagner, le nombre minimal de dplacement est :

avec 3 disques : 7 dplacements
     4   "     : 7*2 +1 = 15 dplacements
     5   "     : 15*2+1 = 31     "
     6   "     : 31*2+1 = 63     "
     etc
Les supports blancs seront mis en arrire-plan
*)


procedure Animation;
Var i : Integer;
begin
  For i := 1 to 20 * Nb_Disques do
  begin
    Form1.Color := Random(MaxInt);
    Form1.Refresh;
  end;
end;

procedure Affiche_Resultat(Combien : Byte);
VAR Nombre_Mini : Integer;
    Couleur : TColor;
begin
  Case Combien of
    3 : Nombre_Mini := 7;
    4 : Nombre_Mini := 15;
    5 : Nombre_Mini := 31;
    6 : Nombre_Mini := 63;
  end;
  If Nb_Essais = Nombre_Mini then
  begin
    Couleur := Form1.Color;
    Animation;
    Form1.Color := Couleur;
    Form1.BAbandon.Enabled := false;
    MessageDlg('BRAVO',mtInformation,[mbOk],0)
  end
  else MessageDlg('GOOG, but we could succed in '+
  IntToStr(Nombre_Mini)+' trys',mtWarning,[mbOk],0);
end;

 (* Selectionner tous les disques sauf le plus petit *)
procedure TForm1.CDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := ((sender as tshape).Hint[2]= 'X')
  and ((Source as TShape).Width < (Sender as TShape).width)
  and Jeu_Possible;
end;
 (* Selectionner tous les disques sauf le plus petit *)
procedure TForm1.CDragDrop(Sender, Source: TObject; X, Y: Integer);
Var Moved, Still : TShape;
begin
  Inc(Nb_Essais);
  Label_Essais.Caption := IntToStr(Nb_Essais);
  Moved := Source as TShape;
  Still := Sender as Tshape;
  Moved.Top := Still.top - Moved.Height;
  Moved.Left := Still.Left + (Still.Width div 2) - (Moved.Width div 2);
  With FindComponent(Moved.Hint[1])
  as TShape do
  begin
    if Name <= Dernier_Disque then DragMode := dmAutomatic;
    Hint := Hint[1] +'X'
  end;
  Moved.Hint := Still.Name + 'X';
  Still.Hint := Still.Hint[1] + Moved.Name;
  Still.DragMode := dmManual;
  if (L.Hint = 'XX') and (M.Hint= 'XX') then
  begin
    Jeu_Possible := False;
    A.DragMode := dmManual;
    Affiche_Resultat(Nb_Disques);
(*    BDemarrer.Caption := '&Rejouer'; *)
    BDemarrer.Enabled := True;
    SpinEdit1.Enabled := True;
    Choisit_Niveau(3);
    Nb_Essais := 0;
    Label_Essais.Caption := '';
  end;
end;

procedure TForm1.BDemarrerClick(Sender: TObject);
begin
  Jeu_Possible := True;
  A.DragMode := dmAutomatic;
  BDemarrer.Enabled  := False;
  SpinEdit1.Enabled := False;
  BAbandon.Enabled := true;
end;

procedure TForm1.Initialise_Hints;
begin
  A.Hint := 'BX';
  B.Hint := 'CA';
  C.Hint := 'DB';
  D.Hint := 'EC';
  E.Hint := 'FD';
  F.Hint := 'LE';
  L.Hint := 'XF';
  M.Hint := 'XX';
  R.Hint := 'XX';
end;

(* Il y a srement une mthode plus lgante, peut-tre en crant dynamiquement *)
(* un tableau de TShape, si quelqu'un veut bien le faire, qu'il m'en envoie une *)
(* version, je suis preneur !                                                   *)

procedure TForm1.Choisit_Niveau(Lequel : Byte);
Const h = 24; (* hauteur entre disques *)
begin
  A.Left := L.Left + (L.Width-A.Width) DIV 2;
  B.Left := A.Left -8;
  C.Left := B.Left -8;
  D.Left := C.Left -8;
  E.Left := D.Left -8;
  F.Left := E.Left -8;
  Initialise_Hints;
  Case Lequel of
    3 : begin
          A.Top := L.Top - h*3;
          A.Visible := True;
          B.Visible := True;
          C.Visible := True;
          D.Visible := False;
          E.Visible := False;
          F.Visible := False;
          C.Hint := 'LB';
          L.Hint := 'XC';
          Dernier_Disque := 'C';
        end;
    4 : begin
          A.Top := L.Top - h*4;
          A.Visible := True;
          B.Visible := True;
          C.Visible := True;
          D.Visible := True;
          E.Visible := False;
          F.Visible := False;
          D.Hint := 'LC';
          L.Hint := 'XD';
          Dernier_Disque := 'D';
        end;
    5 : begin
          A.Top := L.Top - h*5;
          A.Visible := True;
          B.Visible := True;
          C.Visible := True;
          D.Visible := True;
          E.Visible := True;
          F.Visible := False;
          E.Hint := 'LD';
          L.Hint := 'XE';
          Dernier_Disque := 'E';
        end;
    6 : begin
          A.Top := L.Top - h*6;
          A.Visible := True;
          B.Visible := True;
          C.Visible := True;
          D.Visible := True;
          E.Visible := true;
          F.Visible := true;
          F.Hint := 'LE';
          L.Hint := 'XF';
          Dernier_Disque := 'F';
        end;
  end;
  B.Top := A.Top + h;
  C.Top := B.Top + h;
  D.Top := C.Top + h;
  E.Top := D.Top + h;
  F.Top := E.Top + h;
  SpinEdit1.Text := IntTostr(Lequel);
end;

procedure TForm1.SpinEdit1Change(Sender: TObject);
begin
  Nb_Disques := StrToInt(SpinEdit1.Text);
  Choisit_Niveau(Nb_disques);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Nb_Essais := 0;
  Choisit_Niveau(3);   (* le niveau le plus facile *)
  nb_disques := 3;
  Randomize;
end;
(***** La solution est donne par 1 programme en Turbo-Prolog ! + ligne commande  *)
procedure TForm1.BSolutionClick(Sender: TObject);
Var Prog_Solution : String;
begin
  Prog_Solution := 'SOLHANOI.EXE '+IntToStr(Nb_Disques)+#0;
  WinExec(@Prog_Solution[1],SW_SHOWNORMAL)
end;

procedure TForm1.BBAideClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_CONTENTS,0);
end;

Const Select1 : Integer = 0;   (* vignette slectionne au dmarrage *)

procedure TForm1.Timer1Timer(Sender: TObject);
Const
  Clip1X : integer = 40;    (* largeur d'une vignette *)
  Clip1Y : integer = 40;    (* hauteur     "  "       *)
Var

NumClip1 : Integer;    (* nombre de vignettes *)
x,y      : Integer;    (* position de la vignette *)
W, H     : Integer;    (* dimensions corriges *)
  begin
      W := Image1.Width - 1;   (*  cause de l'paisseur de la grille *)
      H := Image1.Height - 1;
      NumClip1 := (Image1.Width div Clip1X) *
      (Image1.Height div Clip1Y);
  (* on boucle *)
      if Select1 >= NumClip1 then Select1 := 0;
      X := (Select1 * Clip1X) mod W;
      Y := Clip1Y * ((Select1 * Clip1X) div W);
      Canvas.CopyRect(Rect(Image2.Left,Image2.Top,Image2.Left+Clip1X,
      Image2.Top+Clip1Y),Image1.Canvas, Rect(X,Y,X+Clip1X,Y+Clip1Y));
      Inc(Select1);
  end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Application.HelpCommand(Help_Quit,0);
end;

procedure TForm1.BAbandonClick(Sender: TObject);
begin
  Jeu_Possible := False;
  A.DragMode := dmManual;
(*  BDemarrer.Caption := '&Rejouer'; *)
  BDemarrer.Enabled := True;
  SpinEdit1.Enabled := True;
  Nb_Essais := 0;
  Label_Essais.Caption := '';
  Choisit_Niveau(Nb_disques);
  BAbandon.enabled := false;
end;

end.
